#!/usr/bin/perl -w
use strict;

# Micompare.pm 1.1, 2017

#######################################################################
#  Extra subroutines for comparative analysis of Mitarget objects     #
#######################################################################
my $PATH = `which seedVicious`;
$PATH =~ s/seedVicious$//g;
$PATH =~ s/\/+$//g;
chomp $PATH;


sub seedPairs{
	my $self = $_[0];
	bless($self,"Mitargets");
	my %transcript = ( );    
	for(keys %{$self}){
		if(scalar @{$self->{$_}->getPositions} < 2){ # at least two target sites
		}else{
			foreach my $pos (@{$self->{$_}->getPositions}){
				push(@{$transcript{$self->{$_}->getTranscriptName}},$self->{$_}->getMiRNA.">".$pos); 
			}
		}
	}

	my $output = "\n####################################################################\n";
	$output .=  "# Pairs of microRNA target sites and minimum distance interactions #\n";
	$output .= "####################################################################\n\n";
	$output .= "transcript\tmicroRNA\ttarget_sites\tmin_distance\n";

	return $output.pairsAndDistances(\%transcript)."\n";
}

sub pairsAndDistances{
	my %transcripts = %{$_[0]};
	my @output = ( );
	foreach my $tr (keys %transcripts){
	my %mir_sites = ( );
	foreach my $miR (@{$transcripts{$tr}}){
		my @pair = split(">",$miR);
		push(@{$mir_sites{$pair[0]}},$pair[1]);
	}
		for(keys %mir_sites){
			push(@output,$tr."\t".$_."\t".scalar @{$mir_sites{$_}}."\t".min_difference_array($mir_sites{$_}));
		}
	}
	return join("\n",@output);
}

sub min_difference_array{
	my $min_dist = 9999999999999999999999999999999999999999999999999999999999999999999999999999;
	for(my $i=0; $i<((scalar @{$_[0]})-1); $i++){
		for(my $j=$i+1; $j<(scalar @{$_[0]}); $j++){
			if(abs(${$_[0]}[$i] - ${$_[0]}[$j]) < $min_dist){
				$min_dist = abs(${$_[0]}[$i] - ${$_[0]}[$j]);
			}
		}
	}
	return $min_dist;
}

sub micompareTargets{
	my $self = $_[0];
	bless($self,"Mitargets");

	my %transcript = ( );
	for(keys %{$self}){
		if(scalar @{$self->{$_}->getPositions} < 1){ 
		}else{
			push(@{$transcript{$self->{$_}->getTranscriptName}},$self->{$_}->getMiRNA); 
		}
	}

	my $output = "\n################################################################\n";
	$output .=  "#  Comparative analysis of microRNA-transcript interactions    #\n";
	$output .= "################################################################\n\n";
	$output .= "transcript\ttranscript\tcommon_miRs\n";

	# Should not be necessary, since MiTargets objects already account for this
	#     for (keys %transcript){ # Keep only unique microRNA interactions
	# 	@{$transcript{$_}} = removeDup(@{$transcript{$_}});
	#     }

	foreach my $tr1 (keys %transcript){ # pairwise comparison of transcripts
		foreach my $tr2 (keys %transcript){
			unless($tr1 eq $tr2){
			$output .= $tr1."\t".$tr2."\t". scalar common_unique_elements(\@{$transcript{$tr1}},\@{$transcript{$tr1}}) ."\t" . "[".join("|",common_unique_elements(\@{$transcript{$tr1}},\@{$transcript{$tr1}}))."]"."\n";
			}
		}
	}
	return $output;
}

sub micompareTargetSites{
	my $self = $_[0];
	bless($self,"Mitargets");

	my %transcript = ( );
	for(keys %{$self}){
		if(scalar @{$self->{$_}->getPositions} < 1){ 
		}else{
			foreach my $pos (@{$self->{$_}->getPositions}){
				push(@{$transcript{$self->{$_}->getTranscriptName}},$self->{$_}->getMiRNA . ">". ${$self->{$_}->getPositionsAli}{$pos}); # position in alignment
			}
		}
	}

	my $output = "\n################################################################\n";
	$output .=  "#   Comparative analysis of conserved microRNA target sites    #\n";
	$output .= "################################################################\n\n";
	$output .= "transcript\ttranscript\tcommon_miRs\n";

	foreach my $tr1 (keys %transcript){ # pairwise comparison of transcripts
		foreach my $tr2 (keys %transcript){
			unless($tr1 eq $tr2){
			$output .= $tr1."\t".$tr2."\t". scalar common_unique_elements(\@{$transcript{$tr1}},\@{$transcript{$tr1}}) ."\t" . "[".join("|",common_unique_elements(\@{$transcript{$tr1}},\@{$transcript{$tr1}}))."]"."\n";
			}
		}
	}
	return $output;
}

sub gainsLossesParsimony{
	my $self = $_[0];
	bless($self,"Mitargets");
	my $tree = $_[1];   

	my %transcript = ( );
# 	my %non_target_transcripts = ( );
	for(keys %{$self}){
		if(scalar @{$self->{$_}->getPositions} < 1){ # transcripts with no targets
			my @notarget = split('@',$_);
# 			$non_target_transcripts{$notarget[1]}=1;
		}else{
			foreach my $pos (@{$self->{$_}->getPositions}){
			push(@{$transcript{$self->{$_}->getTranscriptName}},$self->{$_}->getMiRNA . ">". ${$self->{$_}->getPositionsAli}{$pos}); # position in alignment
			}
		}
	}


	# NEED TO INCLUDE ALIGNEMNT SITE INFO HERE!!!!

	my $output = "\n#####################################################################\n";
	$output .=  "# Parminony-based reconstruction of ancestral microRNA target sites #\n";
	$output .= "#-------------------------------------------------------------------#\n";
	$output .= "#   Phylip dollop 3.695, unweighted Dollo parsimony on user tree    #\n";
	$output .= "#####################################################################\n\n";

	# LIST OF TARGET SITES IN ALIGNMENT
	my %all_targets_alignment = ( );
	for(keys %transcript){
		foreach my $target (@{$transcript{$_}}){
			unless(exists $all_targets_alignment{$target}){
			$all_targets_alignment{$target}=1;
			}
		}
	}
	my @all_targets = keys %all_targets_alignment; # fix order of elements in an array
	@all_targets = sort_targets_by_position(\@all_targets);

	####
	my %phylip_infile = ( );
	for(keys %transcript){
		my %tr_check = ( );
		for(@{$transcript{$_}}){$tr_check{$_} = 1;}
		my $presence = "";
		for(my $i=0; $i<(scalar @all_targets);$i++){
			if(exists $tr_check{$all_targets[$i]}){
				$presence .= "1";
			}else{
				$presence .= "0";
			}
		}
		$phylip_infile{$_}=$presence;
	}
	# Add empty transcripts (those with no target sites)
	my $no_target_sites = "";
	for(my $i=0; $i<(scalar @all_targets);$i++){
		$no_target_sites .= "0";
	}
# 	for(keys %non_target_transcripts){
# 		unless(exists $phylip_infile{$_}){
# 			$phylip_infile{$_}=$no_target_sites;
# 		}
# 	}
	my %transcript_2_alias = ( );
	my $name = 0;
	for(keys %phylip_infile){
		$name++;
		$transcript_2_alias{$_} = "tr".$name;
	}
	my %alias_2_transcript = reverse %transcript_2_alias;
		
# 	if((keys %transcript_2_alias) == (keys %non_target_transcripts)){ # If no targets detected, do not compute the tree
# 		return "No target sites detected in alignment\n";
# 	}


	# ALIAS IN TREE
	my $alias_tree = $tree;
	for(keys %transcript_2_alias){
		if($alias_tree =~ /(.*[\,\(\)])($_)([\,\(\)].*)/){
			$alias_tree = $1.$transcript_2_alias{$2}.$3;
		}
	}

	my $PID = $$;
	# Phylip input file
	open(TEMP_Phylip,">$PID._temp_phylip");
	print TEMP_Phylip "   ". (scalar keys %phylip_infile) ."   ". (scalar @all_targets) ."\n";
	for(keys %alias_2_transcript){
	my $spacer = "";
	for(my $i=0; $i<(10 - length($_)); $i++){$spacer .= " ";}
		print TEMP_Phylip $_ . $spacer . $phylip_infile{$alias_2_transcript{$_}}."\n";
	}
	close TEMP_Phylip;
	# input tree
	open(TEMP_tree,">$PID._temp_tree");
	print TEMP_tree $alias_tree ."\n";
	close TEMP_tree;

	# dollop parameters
	open(TEMP_dollop,">$PID._temp_dollop");
	print TEMP_dollop "$PID._temp_phylip\n";
	print TEMP_dollop "U\n2\n5\n6\n"; # options
	print TEMP_dollop "Y\n$PID._temp_tree\n"; # options

	print TEMP_dollop "\n";
	close TEMP_dollop;
	if(-e "outfile"){system("/bin/rm outfile\n");} #remove previous phylip input/output files
	if(-e "outtree"){system("/bin/rm outtree\n");} #remove previous phylip input/output files
	if(-e "intree"){system("/bin/rm intree\n");} #remove previous phylip input/output files
	if(-e "infile"){system("/bin/rm infile\n");} #remove previous phylip input/output files
	
	
	unless($^O eq "darwin"){
		system("$PATH/bin/dollop < $PID._temp_dollop 2>&1 > /dev/null\n");
	}else{ # Support for MacOS X
		system("$PATH/binm/dollop < $PID._temp_dollop 2>&1 > /dev/null\n");
	}

	# READ PHYLIP DOLLOP OUTOUT FILE
	open(PHYLIP_outfile,"outfile");
	my $phylip_outfile = join("",<PHYLIP_outfile>);
	close PHYLIP_outfile;
	system("/bin/rm $PID._temp_phylip $PID._temp_dollop $PID._temp_tree outfile");

	my ($tree_dollop_1,$tree_dollop_2, $number_changes, $branch2changes) = format_output_dollop($phylip_outfile,\%alias_2_transcript); # returns a simplified Newick tree and extra information from the dollop output information.

	my %branch2lists = ( );
	for(keys %{$branch2changes}){
		my @BINS = split("",${$branch2changes}{$_});
		for(my $i=0; $i< scalar @BINS; $i++){
			if($BINS[$i] eq '1'){
			$branch2lists{$_} .= "|+".$all_targets[$i]
			}elsif($BINS[$i] eq '0'){
			$branch2lists{$_} .= "|-".$all_targets[$i]
			}else{}
		}
	}

	# List per branch
	for(keys %branch2lists){
		if($_ =~ /^tr/){ # LEAVES
			$tree_dollop_2 =~ s/$_\,/$_ $branch2lists{$_}\,/g;
			$tree_dollop_2 =~ s/$_\)/$_ $branch2lists{$_}\)/g;
		}else{  # NODES
			$tree_dollop_2 =~ s/\#$_\,/ $branch2lists{$_}\,/g;
			$tree_dollop_2 =~ s/\#$_\)/ $branch2lists{$_}\)/g;
		# 	        $tree_dollop_2 =~ s/\#$_\;/ $branch2lists{$_}\;/g; # LABEL IN ROOT IS CURRENTLY DISABLED (not necessary for gain/loss analysis)
			$tree_dollop_2 =~ s/\#$_\;/\;/g;                        # REMOVE root node
		}
	}

		# Translate alias to original names
		for(keys %alias_2_transcript){
			$tree_dollop_2 =~ s/$_\,/$alias_2_transcript{$_}\,/g;
			$tree_dollop_2 =~ s/$_ /$alias_2_transcript{$_} /g;
			$tree_dollop_2 =~ s/$_\)/$alias_2_transcript{$_}\)/g;
		}

	$output .= "Number of changes required = ".$number_changes."\n\n".$tree_dollop_1 ."\n". $tree_dollop_2 ."\n";   
	return $output;
}



sub gainsLossesParsimonyHTML{
	my $self = $_[0];
	bless($self,"Mitargets");
	my $tree = $_[1];   

	my %transcript = ( );
# 	my %non_target_transcripts = ( );
	for(keys %{$self}){
		if(scalar @{$self->{$_}->getPositions} < 1){ # transcripts with no targets
			my @notarget = split('@',$_);
# 			$non_target_transcripts{$notarget[1]}=1;
		}else{
			foreach my $pos (@{$self->{$_}->getPositions}){
				push(@{$transcript{$self->{$_}->getTranscriptName}},$self->{$_}->getMiRNA . ">". ${$self->{$_}->getPositionsAli}{$pos}); # position in alignment
			}
		}
	}

	# LIST OF TARGET SITES IN ALIGNMENT
	my %all_targets_alignment = ( );
	for(keys %transcript){
		foreach my $target (@{$transcript{$_}}){
			unless(exists $all_targets_alignment{$target}){
				$all_targets_alignment{$target}=1;
			}
		}
	}
	my @all_targets = keys %all_targets_alignment; # fix order of elements in an array
	@all_targets = sort_targets_by_position(\@all_targets);

	####
	my %phylip_infile = ( );
	for(keys %transcript){
		my %tr_check = ( );
		for(@{$transcript{$_}}){$tr_check{$_} = 1;}
		my $presence = "";
		for(my $i=0; $i<(scalar @all_targets);$i++){
			if(exists $tr_check{$all_targets[$i]}){
				$presence .= "1";
			}else{
				$presence .= "0";
			}
		}
		$phylip_infile{$_}=$presence;
	}
	# Add empty transcripts (those with no target sites)
	my $no_target_sites = "";
	for(my $i=0; $i<(scalar @all_targets);$i++){
		$no_target_sites .= "0";
	}
# 	for(keys %non_target_transcripts){
# 		unless(exists $phylip_infile{$_}){
# 			$phylip_infile{$_}=$no_target_sites;
# 		}
# 	}
	my %transcript_2_alias = ( );
	my $name = 0;
	for(keys %phylip_infile){
		$name++;
		$transcript_2_alias{$_} = "tr".$name;
	}
	my %alias_2_transcript = reverse %transcript_2_alias;

	# ALIAS IN TREE
	my $alias_tree = $tree;
	for(keys %transcript_2_alias){
		if($alias_tree =~ /(.*[\,\(\)])($_)([\,\(\)].*)/){
			$alias_tree = $1.$transcript_2_alias{$2}.$3;
		}
	}


	my $PID = $$;
	# Phylip input file
	open(TEMP_Phylip,">$PID._temp_phylip");
	print TEMP_Phylip "   ". (scalar keys %phylip_infile) ."   ". (scalar @all_targets) ."\n";
	for(keys %alias_2_transcript){
		my $spacer = "";
		for(my $i=0; $i<(10 - length($_)); $i++){$spacer .= " ";}
		print TEMP_Phylip $_ . $spacer . $phylip_infile{$alias_2_transcript{$_}}."\n";
	}
	close TEMP_Phylip;
	# input tree
	open(TEMP_tree,">$PID._temp_tree");
	print TEMP_tree $alias_tree ."\n";
	close TEMP_tree;



	# dollop parameters
	open(TEMP_dollop,">$PID._temp_dollop");
	print TEMP_dollop "$PID._temp_phylip\n";
	print TEMP_dollop "U\n2\n5\n6\n"; # options
	print TEMP_dollop "Y\n$PID._temp_tree\n"; # options

	print TEMP_dollop "\n";
	close TEMP_dollop;
	mkdir "$PID";	
	if(-e "$PID/outfile"){system("/bin/rm $PID/outfile\n");} #remove previous phylip input/output files
	if(-e "$PID/outtree"){system("/bin/rm $PID/outtree\n");} #remove previous phylip input/output files
	if(-e "$PID/intree"){system("/bin/rm $PID/intree\n");} #remove previous phylip input/output files
	if(-e "$PID/infile"){system("/bin/rm $PID/infile\n");} #remove previous phylip input/output files
	system("cd $PID; $PATH/cgi-bin/bin/dollop < $PID._temp_dollop 2>&1 > /dev/null\n");

	# READ PHYLIP DOLLOP OUTOUT FILE
	open(PHYLIP_outfile,"$PID/outfile");
	my $phylip_outfile = join("",<PHYLIP_outfile>);
	close PHYLIP_outfile;
	system("/bin/rm -r $PID._temp_phylip $PID._temp_dollop $PID._temp_tree $PID");

	my ($tree_dollop_1,$tree_dollop_2, $number_changes, $branch2changes) = format_output_dollop($phylip_outfile,\%alias_2_transcript); # returns a simplified Newick tree and extra information from the dollop output information.

	my $print_tree = $tree_dollop_2;
	# Translate alias to original names in Printed output tree
	for(keys %alias_2_transcript){
		$print_tree =~ s/$_\,/$alias_2_transcript{$_}\,/g;
		$print_tree =~ s/$_ /$alias_2_transcript{$_} /g;
		$print_tree =~ s/$_\)/$alias_2_transcript{$_}\)/g;
	}
	my %branch2lists = ( );
	for(keys %{$branch2changes}){
		my @BINS = split("",${$branch2changes}{$_});
		for(my $i=0; $i< scalar @BINS; $i++){
			if($BINS[$i] eq '1'){
				$branch2lists{$_} .= "|+".$all_targets[$i]
			}elsif($BINS[$i] eq '0'){
				$branch2lists{$_} .= "|-".$all_targets[$i]
			}else{}
		}
	}
	# Translate alias to original names
	for(keys %alias_2_transcript){
		$tree_dollop_2 =~ s/$_\,/$alias_2_transcript{$_}\,/g;
		$tree_dollop_2 =~ s/$_ /$alias_2_transcript{$_} /g;
		$tree_dollop_2 =~ s/$_\)/$alias_2_transcript{$_}\)/g;
	}

	my $output_1 = ""; # output scalar


	my @sorted_array_of_leaves = sorted_leaves_newick($alias_tree);
	my @sorted_array_of_inner_nodes = ( );
	for(keys %branch2lists){unless($_ =~ /^tr/){push(@sorted_array_of_inner_nodes,$_)}}
	@sorted_array_of_inner_nodes = sort @sorted_array_of_inner_nodes;
	if($sorted_array_of_inner_nodes[0]==1){shift @sorted_array_of_inner_nodes;} #remove root note if present (not informative)
	my @sorted_nodes = (@sorted_array_of_leaves,@sorted_array_of_inner_nodes);

	$output_1 .= "<TABLE>\n";
	$output_1 .= "<TR><TH>Branch</TH><TH>Gains</TH><TH>Losses</TH></TR>\n";
	for(@sorted_nodes){
		if(exists $alias_2_transcript{$_}){
			$output_1 .= "<TR><TD>".$alias_2_transcript{$_}."</TD><TD>".gainsAtBranchNumber($branch2lists{$_})."</TD><TD>".lossesAtBranchNumber($branch2lists{$_})."</TD></TR>\n";
		}else{
			$output_1 .= "<TR><TD>#".$_."</TD><TD>".gainsAtBranchNumber($branch2lists{$_})."</TD><TD>".lossesAtBranchNumber($branch2lists{$_})."</TD></TR>\n";
		}
	}
	$output_1 .= "</TABLE>\n";

	my $output_2 = ""; # output scalar

	$output_2 .= "<TABLE>\n";
	$output_2 .= "<TR><TH>Branch</TH><TH>Gains</TH><TH>Losses</TH></TR>\n";
	

	
	for(@sorted_nodes){
		if(exists $alias_2_transcript{$_}){
			$output_2 .= "<TR><TD>".$alias_2_transcript{$_}."</TD><TD>".gainsAtBranch($branch2lists{$_})."</TD><TD>".lossesAtBranch($branch2lists{$_})."</TD></TR>\n";
		}else{
			$output_2 .= "<TR><TD>#".$_."</TD><TD>".gainsAtBranch($branch2lists{$_})."</TD><TD>".lossesAtBranch($branch2lists{$_})."</TD></TR>\n";
		}

	}
	$output_2 .= "</TABLE>\n";
	
	return ($output_1,$output_2,$print_tree);
}

sub sorted_leaves_newick{
	my @output = split /[\(\)\,\;]+/, $_[0];
	shift @output;
	pop @output;	
	return @output;
}

sub gainsAtBranch{
	my @changes = split /\|/,$_[0];
	my @outchanges = ( );
	for(@changes){
		if($_ =~ /^\+/){
			$_ =~ s/^\+//g;
			$_ =~ s/\>/(/g;
			$_ .= ")";
			push(@outchanges,$_);
		}
	}
	return join("; ",@outchanges);
}


sub lossesAtBranch{
	my @changes = split /\|/,$_[0];
	my @outchanges = ( );
	for(@changes){
		if($_ =~ /^\-/){
			$_ =~ s/^\-//g;
			$_ =~ s/\>/(/g;
			$_ .= ")";
			push(@outchanges,$_);
		}
	}
	return join("; ",@outchanges);
}

sub gainsAtBranchNumber{
	my @changes = split /\|/,$_[0];
	my @outchanges = ( );
	for(@changes){
		if($_ =~ /^\+/){
			$_ =~ s/^\+//g;
			$_ =~ s/\>/(/g;
			$_ .= ")";
			push(@outchanges,$_);
		}
	}
	return scalar @outchanges;
}


sub lossesAtBranchNumber{
	my @changes = split /\|/,$_[0];
	my @outchanges = ( );
	for(@changes){
		if($_ =~ /^\-/){
			$_ =~ s/^\-//g;
			$_ =~ s/\>/(/g;
			$_ .= ")";
			push(@outchanges,$_);
		}
	}
	return scalar @outchanges;
}



sub format_output_dollop{
# returns a simplified and a detailed Newick tree with the dollop output information.
	my $outfile_dollop = $_[0];
	my $alias2tr = "noalias";
	if($_[1]){$alias2tr=$_[1]}

	my $number_of_changes = 0;
	my $tree_counts = "empty";
	my $tree_lists = "empty";

	$outfile_dollop = join("\t",split("\n",$outfile_dollop));
	# Replace \n by \t. All output in one line. This is possible since phylip does not use TAB in their outputs (no conflict)
	if($outfile_dollop =~ /User\-defined\stree\:\t(.+)requires\sa\stotal\sof\s+([0-9]+)\..+\t(root.+)$/){
	$number_of_changes = $2;
	$tree_counts = remove_multiple_tabs($1);

	my $binary_table = remove_multiple_tabs($3);
	$binary_table =~ s/ \t         //g; # merge lines from the same OTU


	my @Binary_array = split /\t/, $binary_table;
	my %binary_branch = ( );
	my %branch_to_changes = ( );
	my %branch_to_binaries = ( );
	for(@Binary_array){
		if($_ =~ /[\sA-Za-z0-9]+\s+([A-Za-z0-9]+)\s+[yesno]+\s+([01\.\s]+)/){
		$binary_branch{$1} = $2;
		$binary_branch{$1} =~ s/ //g; # remove gaps
		}
	}
	for(keys %binary_branch){
		$branch_to_changes{$_}=changes_from_binary($binary_branch{$_});
		$branch_to_binaries{$_}=$binary_branch{$_};
	}


	$tree_counts = displaytree2newick($tree_counts);
	$tree_lists = $tree_counts; # Basic tree is the same for both outputs

	# Counts per branch
	for(keys %branch_to_changes){
		if($_ =~ /^tr/){ # LEAVES
			$tree_counts =~ s/$_\,/$_ $branch_to_changes{$_}\,/g;
			$tree_counts =~ s/$_\)/$_ $branch_to_changes{$_}\)/g;
		}else{  # NODES
			$tree_counts =~ s/\#$_\,/ $branch_to_changes{$_}\,/g;
			$tree_counts =~ s/\#$_\)/ $branch_to_changes{$_}\)/g;
		# 	        $tree_counts =~ s/\#$_\;/ $branch_to_changes{$_}\;/g; # LABEL IN ROOT IS CURRENTLY DISABLED (not necessary for gain/loss analysis)
			$tree_counts =~ s/\#$_\;/\;/g;                        # REMOVE root node
		}
	}

	# Translate alias to original names
	unless($alias2tr eq "noalias"){
		for(keys %{$alias2tr}){
			$tree_counts =~ s/$_\,/${$alias2tr}{$_}\,/g;
			$tree_counts =~ s/$_ /${$alias2tr}{$_} /g;
			$tree_counts =~ s/$_\)/${$alias2tr}{$_}\)/g;
		}
	}

	return ($tree_counts,$tree_lists,$number_of_changes,\%branch_to_binaries);

	}else{
		die "Error: dollop (phylip) outfile could not be parsed\n";
	}
}


# # convert a phylip output display tree (with tabs instead of returns) into a newick tree. Recursive function.
# taxoin name starts with 't' or '('
sub displaytree2newick{
	my $tree = $_[0];
	my @leaves = $tree =~ /\+\-+([t\(][A-Za-z0-9\#\(\)\,]+)\s*\t/g;
	my %distance_from_root = ( );
	foreach my $leave (@leaves){
		my $OTU_regex = $leave;
		$OTU_regex =~ s/\(/\\\(/g;
		$OTU_regex =~ s/\)/\\\)/g;
		if($tree =~ /\t([\+! 0-9\-]+\+)\-+$OTU_regex *\t/){
			$distance_from_root{$leave}=length($1);
		}else{
			die "Error: wrong tree format in \'displaytree2newick\' subroutine\n";
		}
	}

	# walk in pairs, ordered
	for(my $i=1; $i<(scalar @leaves);$i++){
		if($distance_from_root{$leaves[$i-1]} == $distance_from_root{$leaves[$i]}){ # evaluate ONLY pairs at the same distance to the root
			my $out1_regex = $leaves[$i-1];
			$out1_regex =~ s/\(/\\\(/g;
			$out1_regex =~ s/\)/\\\)/g;      
			my $out2_regex = $leaves[$i];
			$out2_regex =~ s/\(/\\\(/g;
			$out2_regex =~ s/\)/\\\)/g;

			if($tree =~ /\t[\s0-9\-\!\+]*?\+\-+$out1_regex\s*\t[\s\!\+\-0-9]*?\+\-+([0-9]+)\s+\t[\s0-9\!\+\-]*?\+\-+$out2_regex\s+\t/){
				my $node = $1;
				$tree =~ s/\+\-+$out1_regex *\t/\t/g;
				$tree =~ s/\+\-+$out2_regex *\t/\t/g;
				$tree =~ s/\+\-+$node /\+\-\-\($out1_regex\,$out2_regex\)\#$node/g;
				$tree =~ s/\\\)/\)/g; # replace back to original brackets, no regex
				$tree =~ s/\\\(/\(/g; # replace back to original brackets, no regex
				$tree =~ s/\t[\s\!]+\t/\t/g;		  
				return displaytree2newick($tree);
			}
		}
	}

	# ROOT
	if($tree =~ /\+\-+(.*?) *\t\-\-1.*?\t\s+\+\-+(.*?)\s/){
		return "(".$1.",".$2.")#1;";
	}else{
		die "Error: error parsing tree in \'displaytree2newick\' subroutine\n";
	}
}
    
sub changes_from_binary{
      my $gains = $_[0] =~ tr/1//;
      my $losses = $_[0] =~ tr/0//;
      return "\+".$gains."|\-".$losses;
}


sub remove_multiple_tabs{ # remove '\t' repeated in tandem. Recursive function!
	my $inline = $_[0];
	if($inline =~ /\t\t/){
		$inline =~ s/\t\t/\t/g;
		remove_multiple_tabs($inline);
	}else{
		return $inline;
	}
}

sub removeDup{ # Remove duplicated elements in Array
	my @elements = @_;
	my %seen = ();
	my @uniq = ();
	foreach my $item (@elements) {
	unless ($seen{$item}) {
		# if we get here, we have not seen it before
		$seen{$item} = 1;
		push(@uniq, $item);
		}
	}
	return @uniq;
}

sub common_unique_elements{
# input two arrays of unique elements
# outputs common elements between the two arrays
	my %array = ( );
	my @output = ( );
	for(@{$_[0]}){
		$array{$_}=1;
	}
	for(@{$_[1]}){
		if(exists $array{$_}){
			push(@output,$_);
		}
	}
	return @output;
}

sub sort_targets_by_position{
	my @input = @{$_[0]};
	my %hash_temp = ( );
	for(@input){
		if($_ =~ />([0-9]+)/){
			$hash_temp{$_}=$1;
		}else{
			die "ERROR: wrong target formating: $_";	
		}
	}

	return sort { $hash_temp{$a} <=> $hash_temp{$b} } keys %hash_temp;
}


1;
