#!/usr/bin/perl -w
use strict;

package Mitargets;

# Mitargets.pm 1.1, 2017


my $PATH = `which seedVicious`;
$PATH =~ s/seedVicious$//g;
$PATH =~ s/\/+$//g;
chomp $PATH;

sub new
{
	my $class = shift;
	if((scalar @_) != 6){die "Invalid number of arguments in Mitargets object construction\n";}
	my ($miRNA,$mature,$is_near,$trName,$trSeq,$opts) = @_;
	my %options = %{$opts};
	# Error: non-RNA nucleotides
	unless($mature =~ /^[ACGUX]+$/){die "Non-RNA nucleotides in microRNA sequence: [$mature]$miRNA";} # Allows X for masked nucleotides
	unless($trSeq =~ /^[ACGUXN-]+$/){die "Non-RNA nucleotides in transcript sequence";} # Allows X for masked nucleotides, and N for undetermined. Allows '-' for gapped alignments
	unless($options{"alignment"}==1){
		$trSeq =~ s/\-//g; # Remove gaps from alignment to treat as ungapped sequence
	}
	my $trSeqGaps = $trSeq;
	$trSeq =~ s/\-//g; # keep two versions fo the aligned transcript, gapped and ungapped
	my $ID = $miRNA."@".$trName;
	my $self = { };

	my $sites_2 = reverseComp(substr($mature,1,1));
	my $sites_3_7 = reverseComp(substr($mature,2,5));
	my $sites_2_7 = $sites_3_7.$sites_2;

	my $site_8 = reverseComp(substr($mature,7,1));
	my @position = ( );
	my %target = ( ); # Fragment of the transcript coxmplementary to (and expanding) the microRNA
	my %paired_positions = ( ); # Paired positions between mature microRNA and the transcript fragments expanded
	my %type = ( );
	while($trSeq =~ /$sites_3_7/gi ){
		unless((pos($trSeq)+2)>length($trSeq)){ # avoid overflow
			my $char_firstA = 0;
			my $char_second = 0;
			my $char_last = 0;
			my $type_of_binding = "5mer";
			if(substr($trSeq,(pos($trSeq)),1) eq $sites_2){
				$type_of_binding = "6mer";
			}
			if(substr($trSeq,(pos($trSeq)-6),1) eq $site_8){
				if($type_of_binding eq "6mer"){
					$char_last = 1;
					$type_of_binding = "7_m8";
				}else{
					$type_of_binding = "off6mer";
				}
			}
			unless(length($trSeq)<=pos($trSeq)){ # Avoid overflow fron 5mers
				if((substr($trSeq,pos($trSeq)+1,1) eq "A")&&($type_of_binding eq "6mer")){
					$char_firstA = 1;
					$type_of_binding = "7_A1";
				}elsif(substr($trSeq,pos($trSeq)+1,1) eq "A"){
					$char_firstA = 1;
				}
			}
			if(($char_firstA == 1)&&($char_last == 1)){
				$type_of_binding = "8mer";
			}
			# Indicate is it is a near target from input

			# Do not report sixmers or off6mers unless the user specifies to do so. Do not report 5mers either
			if((($options{"sixmers"}==0)&&($type_of_binding eq "6mer"))||(($options{"off6mer"}==0)&&($type_of_binding eq "off6mer"))||($type_of_binding eq "5mer")){}else{
			push(@position,(pos($trSeq)+2));
			$type{(pos($trSeq)+2)} = $type_of_binding;
			if((pos($trSeq)-length($mature)+1)>=0){
				$target{(pos($trSeq)+2)} = substr($trSeq,(pos($trSeq)+1-length($mature)+1),length($mature));
			}else{ # If the target exceeds the 5' end of the transcript, need to add Xs at the end of the target
				my $out_of_range = (pos($trSeq)+1-length($mature)+1);
				my $offXs = "";
				for(my $i=0; $i<((-1)*$out_of_range); $i++){$offXs.="X";}
				$target{(pos($trSeq)+2)} = $offXs;
				$target{(pos($trSeq)+2)} .= substr($trSeq,0,length($mature)+$out_of_range);
			}
			$paired_positions{(pos($trSeq)+2)} = pairedNucleotides($target{(pos($trSeq)+2)},$mature);
			}
		}
	}

	# Tag near targets with an asterisk
	if($is_near == 1){
		for(keys %type){
			$type{$_} .= "*";
		}
	}
	
	# Fields for each target/miR/tr
	$self->{_ID} = $ID;
	$self->{_miRNA} = $miRNA;
	$self->{_mature} = $mature;
	$self->{_trName} = $trName;
	$self->{_trSeq} = $trSeq;
	$self->{_positions} = \@position;
	$self->{_type} = \%type;
	$self->{_target} = \%target;
	$self->{_paired} = \%paired_positions;

	# If it is an alignment, generate absolute alignment positions
	if($options{"alignment"}==1){
		my %pos2ali = ( );
		my @seq2gapped_single = ( );
		my $gaps = ($trSeqGaps =~ tr/\-//);
		my @positions = split("",$trSeqGaps);
		my $pos=1;
		for(my $i=0;$i<(scalar @positions);$i++){
			if($positions[$i] eq "-"){
				$pos++;
			}else{
				push(@seq2gapped_single,$pos);
				$pos++;
			}
		}
		my %positionAli;
		for(@position){
			$positionAli{$_} = $seq2gapped_single[$_ - 1];
		}
		$self->{_positionAli} = \%positionAli;
	}
	
	if($options{"energy"}==1){
		my %energy = ( );
		for(@position){
			$energy{$_} = energyDuplexRNA($mature,$paired_positions{$_},$target{$_});
		}
		$self->{_energy} = \%energy;
	}

	bless $self, $class;
	return $self;
}

sub pairedNucleotides
{
	my $output = "";
	for(my $i=0; $i<length($_[0]);$i++){
		if(substr($_[0],$i,1) eq substr(reverseComp($_[1]),$i,1)){
			$output.="|";
		}else{
			$output.=" ";
		}
	}
	return $output;
}

sub displayTargetPairing
{
	my $self = $_[0];
	my $out = "miR  3\' ".reverse($self->getMature())." 5\'\n";
	$out .= "        ".$self->{_paired}->{${$self->{_positions}}[$_[1]]}."\n";
	$out .= "tr   5\' ".$self->{_target}->{${$self->{_positions}}[$_[1]]}." 3\'\n";
	return $out;
}

sub energyDuplexRNA
{
	my $PID = $$;
	open(TEMP_RNAeval,">$PID._temp_rnaeval");
	my $input = $_[0]."&".$_[2]."\n";
	my $left = reverse($_[1]);
	$left =~ s/\|/\(/g;
	$left =~ s/\s/\./g;
	my $right = $_[1];
	$right =~ s/\|/\)/g;
	$right =~ s/\s/\./g;
	
	# PATCH for Unspecific ERROR. TO BE DOUBLE-CHECKED!
	if(length($_[0])==(length($_[2])+1)){
		$left = ".".$left;
	}
	
	$input .= $left."&".$right."\n";
	$input .= "\@\n";
	print TEMP_RNAeval $input;
	close TEMP_RNAeval;
	
	my $energy = "";
	
	unless($^O eq "darwin"){
		$energy = `$PATH/bin/RNAeval < $PID._temp_rnaeval`;
	}else{ # Support for MacOS X
		$energy = `$PATH/binm/RNAeval < $PID._temp_rnaeval`;
	}
	
	
	system("rm $PID._temp_rnaeval");
	if($energy =~ /([\-\s][0-9\.]+)\)/){
		return "$1 Kcal/mol";
	}else{
		return "WARNING: $input";
	}
}

sub setID
{
	my ($self, $ID) = @_;
	$self -> {_ID} = $ID if defined ($ID);
	return $self -> {_ID};
}

sub getID
{
	my $self = $_[0];
	return $self -> {_ID};
}

sub setMiRNA
{
	my ($self, $miRNA) = @_;
	$self -> {_miRNA} = $miRNA if defined ($miRNA);
	return $self -> {_miRNA};
}

sub getMiRNA
{
	my $self = $_[0];
	return $self -> {_miRNA};
}

sub setMature
{
	my ($self, $miMature) = @_;
	unless($miMature =~ /^[ACGTUacgtu]+$/){
		if(exists $self -> {_name}){
			die "ERROR: Not a valid nucleotide in ". $self -> {_name} . " mature sequence\n";
		}else{
			die "ERROR: Not a valid nucleotide in UNNAMED mature sequence\n";
		}
	}
	unless(length($miMature)<30){
		if(exists $self -> {_name}){
			warn "Warning: ". $self -> {_name}. " mature sequence is unexpectedly long (".length($miMature)."nt.)\n";
		}else{
			warn "Warning: UNNAMED mature sequence is unexpectedly long (".length($miMature)."nt.)\n";
		}
	}
	$miMature =~ tr/acgtuT/ACGUUU/;	
	$self -> {_mature} = $miMature if defined ($miMature);
	return $self -> {_mature};
}

sub getMature
{
	my $self = $_[0];
	return $self -> {_mature};
}

sub setTranscript
{
	my ($self, $miTranscript) = @_;
	unless($miTranscript =~ /^[ACGTUacgtu]+$/){
		if(exists $self -> {_trName}){
			warn "Warning: Non-standard nucleotides (ACGT/U) in " . $self -> {_trName}. " will be masked\n";
		}else{
			warn "Warning: Non-standard nucleotides (ACGT/U) in UNNAMED TRANSCRIPT will be masked\n";
		}
	}
	$miTranscript =~ tr/acgtuT/ACGUUU/;
	$miTranscript =~ s/[^ACGU]/X/g; # Masking unwanted characters, non-standard nucleotides
	$self -> {_transcript} = $miTranscript if defined ($miTranscript);
	return $self -> {_transcript};
}

sub getTranscript
{
	my $self = $_[0];
	return $self -> {_transcript};
}

sub setTranscriptName
# Set all set Names to avoid unwanted characters!
{
	my ($self, $miTranscriptName) = @_;
	$self -> {_trName} = $miTranscriptName if defined ($miTranscriptName);
	return $self -> {_trName};
}

sub getTranscriptName
{
	my $self = $_[0];
	return $self -> {_trName};
}

sub setType # Add controls to this and the position subroutine
{
	my ($self, $miType) = @_;
	$self -> {_type} = $miType if defined ($miType);
	return $self -> {_type};
}

sub getType
{
	my $self = $_[0];
	return $self -> {_type};
}

sub setPositions # also control for lims in length?
{
	my ($self, $miPosition) = @_;
	$self -> {_positions} = $miPosition if defined ($miPosition);
	return $self -> {_positions};
}

sub getPositions
{
	my $self = $_[0];
	return $self -> {_positions};
}

sub getPositionsAli
{
	my $self = $_[0];
	return $self -> {_positionAli};
}

sub reverseComp
{
	my $dna = $_[0];
	my $revcomp = reverse($dna);
	$revcomp =~ tr/ACGTUacgtu/UGCAAugcaa/;
	return $revcomp;
}

sub printTargetsOutput
{
    my $self = $_[0];
    my $output = ( );
    if($_[1]==0){ # Table
        if(scalar @{$self->getPositions} < 1){ # if no target sites
        }else{ # if at least one target site
            for(my $p=0; $p<scalar(@{$self->getPositions}); $p++){
                $output .= $self->getMiRNA."\t";
                $output .= $self->getTranscriptName."\t";
                if($self->getTranscriptName =~ /\(R\)$/){
                    $output .= ${$self->getPositions}[$p];
                    $output .= "(";
                    $output .= length($self->{_trSeq}) - ${$self->{_positions}}[$p] + 1;
                    $output .= ")\t";
                }else{
		    if($_[2] == 1){ # ALIGNMENT as input transcript
			  $output .= ${$self->getPositions}[$p]."|".$self->{_positionAli}->{${$self->{_positions}}[$p]}."\t";
                    }else{
			  $output .= ${$self->getPositions}[$p]."\t";
                    }
                }
                $output .= $self->getType->{${$self->{_positions}}[$p]};
                if(exists $self->{_energy}->{${$self->{_positions}}[$p]}){
                    $output .= "\t" . $self->{_energy}->{${$self->{_positions}}[$p]};
                }
                $output .= "\n";
            }
        }
    }elsif($_[1]==1){ # Verbose
        if(scalar @{$self->getPositions} < 1){ # if no target sites
        }else{ # if at least one target site
            for(my $p=0; $p<scalar(@{$self->getPositions}); $p++){
                $output .= ">". $self->getID.":".${$self->getPositions}[$p]."\n";
                $output .= "MicroRNA   = ".$self->getMiRNA."\n";
                $output .= "Transcript = ".$self->getTranscriptName."\n";
                $output .= "Position   = ";
                if($self->getTranscriptName =~ /\(R\)$/){
                    $output .= ${$self->getPositions}[$p];
                    $output .= "(";
                    $output .= length($self->{_trSeq}) - ${$self->{_positions}}[$p] + 1;
                    $output .= ")\n";
                }else{
                    $output .= ${$self->getPositions}[$p]."\n";
                }
                $output .= "Type       = ".$self->getType->{${$self->{_positions}}[$p]}."\n";
                if(exists $self->{_energy}->{${$self->{_positions}}[$p]}){
                    $output .= "Energy     = ". $self->{_energy}->{${$self->{_positions}}[$p]}."\n";
                }
                $output .= "\n".$self->displayTargetPairing($p)."\n\n";
            }
        }
    }else{
        die "Error: output format not recognized\n";
    }
    return $output;
}

sub DESTROY
{

}

sub AUTOLOAD
{
	die "Unknown subroutine\n";
}

1;

